# This script provides codes for R used in the analysis of Meynard & Quinn
# Comments are indicated by the sign # at the beginning of the line. This file can be opened in any text editor. Codes can be either copied and pasted in the R command line, or the file can also be opened within the R script window and run from there. This file was created using Emacs while running R within the same text editor, which can make it a lot easier than using the R script environment. 

# Examples of how to perform partial regressions and spatial analysis using S-Plus can be found in the supplementary material of the paper: Lichtein, Simons, Shriner, & Franzreb (2002)"Spatial Autocorrelation and Autoregressive models in Ecology", Ecological Monographs 72: 445-463 (Ecological Archive M072-007-S1). The procedure for R is basically the same as for S-Plus and can easily be adapted. 

# In this script we provide an example to perform basic Moran's I test on a raw variable and on residuals of a regression, and a detailed example for SEM analysis using our correlation matrix.

# 1) Spatial autocorrelation

# Start by checking autocorrelation in the raw richness data.
# First you need to create a neighborhood matrix with minimum and maximum distances to be considered, shape of the distance function etc.
# Here I used the functions included in the library spdep to build the neighborhood matrices

library(spdep)#loads the library.

# We will assume that you have your geographic coordinates under the names "x" and "y"
neighbour.all <- dnearneigh (as.matrix(dataname[,c("x","y")]), 0, 3000)#identify neighbors, speciefying the x and y coordinates matrix, minimum distance=0 and maximum=3kms.
neighw1 <- nbdists(neighbour.all,as.matrix(all.rich[,c(5,4)]))# Returns euclidean distance given the neighborhood list
neighw1 <- lapply(neighw1, function(x)1/(x^2))# Modifies the previous distances according to a function, in this case 1/(x^2)
neighw1 <- nb2listw(neighbour.all, glist=neighw1)# Finally creates the neighborhood matrix necessary for spatial analysis with its corresponding weights.

# To check for spatial autocorrelation in a raw variable called "richness":
moran.test(richness, neighw1)
# To check for spatial autocorrelation in the residuals of a regression:
# First create the regression with some name:
lm.name <- lm(richness~environment, data=data.name)
# Then perform the test:
lm.morantest(lm.name, neighw1)

# 2) Stepwise selection using AIC and partial regression

# To select variables using a stepwise selection procedure, you could you the function stepAIC or step:

# First build a  model with all the variables:
lm1 <- lm (response ~ predictor1 + I(predictor1^2)+ predictor2 + I(predictor2^2) + predictor3 + I(predictor3^2) + predictor4 + I(predictor4^2), data=data.name)
# Then select the variables:
lm2 <- stepAIC(lm1, direction="backward")

# To carry out a stepwise selection with AIC corrected for small samples, I used a script writen by Spencer Graves that can be downloaded from http://www.prodsyse.com/
# Download the script called stepAICc3.R and load it into your working directory:

source("stepAICc3.R")# This is assuming that you are working in the same directory where you downloaded the code.

# Build a model with all the relevant variables to be considered, including quadratic terms:
lm1 <- lm (response ~ predictor1 + I(predictor1^2)+ predictor2 + I(predictor2^2) + predictor3 + I(predictor3^2) + predictor4 + I(predictor4^2), data=data.name)

# Use the function stepAICc3.R to select the variables. With these settings, if a quadratic term is selected, the non-quadratic is also included
lm2 <- stepAIC.c(lm1, k="AIC.c", hierarchy="include")

# To perform the partial regression, suppose we only had two predictors, and we start by building the full regression with these two:

lm1 <- lm(response ~ predictor1 + predictor2, data=data.name)
# I will store the total variation explained in the response varible by these to predictors in a separate variable:
R.sq.all <- summary(lm1)$adj.r.sq

# Now we are interested in knowing how much variation is explained by predictor1 once the effect of predictor2 has been taken into account, and vice versa.
lm2 <- lm(response ~ predictor1, data=data.name)
R.predictor2 <- R.sq.all - summary(lm2)$adj.r.sq #This is how much variation in the response is explained by predictor2

lm3 <- lm(response ~ predictor2, data=data.name)
R.predictor1 <- R.sq.all - summary(lm3)$adj.r.sq #This is how much variation in the response is explained by predictor1

# 3) Structural Equation Modeling

# After selecting the relevant variables, we permorfed a Structural Equation Modeling analysis using the library sem:
library (sem)
# This library was created by J. Fox, and his website has several useful examples and guidelines: 
# If your data contains different types of variables, you could use the library polycor to calculate the correlation matrix.
# In our case, we had a matrix of continuous data, that were already transformed for normality and scaled. We could therefore use the simple cor function included in the basic package:
correlations <- cor (data.name)

# Here we provide the correlation matrix so that you can follow the example:

correlations <- matrix(c(1,-0.0776620459395211,0.00603989128517595,-0.415716505387213,-0.102118492514785,-0.127050783672849,0.265442173900290,0.134763342272435,0.213742125457316,-0.0776620459395211,1,0.187428889261548,0.155731733541701,-0.00174538675556168,0.0697908996683426,-0.132954747791256,-0.0623767467560623,-0.189907560283323,0.00603989128517595,0.187428889261548,1,-6.57934350194615e-17,-2.41206950068428e-16,0.0688870330752648,-0.160066532043980,-0.182906410801834,-0.253009234470899,-0.415716505387213,0.155731733541701,-6.57934350194615e-17,1,3.20845869611798e-17,-0.100963765316129,-0.431134473514653,-0.296956481052638,-0.326961067648207,-0.102118492514785,-0.00174538675556168,-2.41206950068428e-16,3.20845869611798e-17,1,-0.190263929177813,-0.370962813895642,-0.220492521286324,-0.378602456045603,-0.127050783672849,0.0697908996683426,0.0688870330752648,-0.100963765316129,-0.190263929177813,1,0.192834466907348,0.088310147568352,0.0737476605047341,0.265442173900290,-0.132954747791256,-0.160066532043980,-0.431134473514653,-0.370962813895642,0.192834466907348,1,0.772677715297309,0.881659609291614,0.134763342272435,-0.0623767467560623,-0.182906410801834,-0.296956481052638,-0.220492521286324,0.088310147568352,0.772677715297309,1,0.641974457406936,0.213742125457316,-0.189907560283323,-0.253009234470899,-0.326961067648207,-0.378602456045603,0.0737476605047341,0.881659609291614,0.641974457406936,1), ncol=9, byrow=T)

colnames(correlations) <- c("PC1", "PC3", "nmds1", "nmds4", "nmds5", "logarea", "S.obs", "S.chao1", "shannon")
rownames (correlations) <- c("PC1", "PC3", "nmds1", "nmds4", "nmds5", "logarea", "S.obs", "S.chao1", "shannon")

# This was the initial model specification:
mod.initial <- matrix(c(
                        "PC1 -> nmds1", "alpha1", NA,
                        "PC1 -> nmds4", "alpha2", NA,
                        "PC1 -> nmds5", "alpha3", NA,
                        "PC1 <-> PC1", NA, 1,# the variance of exogenous variables was fixed to 1
                        "PC3 -> nmds1", "beta1", NA,
                        "PC3 -> nmds4", "beta2", NA,
                        "PC3 -> nmds5", "beta4", NA,
                        "PC3 <-> PC3", NA, 1, 
                        "nmds1 <-> nmds1", "err1", NA,
                        "nmds4 <-> nmds4", "err2", NA,
                        "nmds5 <-> nmds5", "err3", NA,
                        "nmds1 -> diversity", "phi1", NA, 
                        "nmds4 -> diversity", "phi2", NA,
                        "nmds5 -> diversity", "phi3", NA,
                        "PC1 -> diversity", "phi4", NA,
                        "PC3 -> diversity", "phi5", NA,
                        "logarea -> nmds1", "gamma1", NA,
                        "logarea -> nmds4", "gamma2",NA,
                        "logarea -> nmds5", "gamma4", NA,
                        "logarea -> diversity", "phi7", NA,
                        "logarea <-> logarea", NA, 1,
                        "diversity <-> diversity", "errordiv", NA,
                        "diversity -> S.obs", NA, 1, #This gives a scale to the latent variable by fixing the contribution of one of the indicators
                        "diversity -> S.chao1", "zeta1", NA,
                        "diversity -> shannon", "zeta2", NA,
                        "S.obs <-> S.obs", "error5", NA,
                        "S.chao1 <-> S.chao1", "error6", NA,
                        "shannon <-> shannon", "error7", NA),
                      ncol=3, byrow=TRUE)
sem.initial <- sem(mod.initial, correlations, 147)
summary(sem.initial)# Notice that the fit of this model is poor, and the error term for S.obs is negative (although the confidence interval does include 0)Grace (2006) cautions about this type of cases, where the correlation between two indicators is really good, but the correlation of the third is good with one variable and poor (<0.7) with the other variable (see Grace 2006, p. 97-98). In this case, S.chao1 is the indicator causing the problem, so in the following step we will repeat this modeling effort excluding S.chao1.

mod.initial <- matrix(c(
                        "PC1 -> nmds1", "alpha1", NA,
                        "PC1 -> nmds4", "alpha2", NA,
                        "PC1 -> nmds5", "alpha3", NA,
                        "PC1 <-> PC1", NA, 1,# the variance of exogenous variables was fixed to 1
                        "PC3 -> nmds1", "beta1", NA,
                        "PC3 -> nmds4", "beta2", NA,
                        "PC3 -> nmds5", "beta4", NA,
                        "PC3 <-> PC3", NA, 1, 
                        "nmds1 <-> nmds1", "err1", NA,
                        "nmds4 <-> nmds4", "err2", NA,
                        "nmds5 <-> nmds5", "err3", NA,
                        "nmds1 -> diversity", "phi1", NA, 
                        "nmds4 -> diversity", "phi2", NA,
                        "nmds5 -> diversity", "phi3", NA,
                        "PC1 -> diversity", "phi4", NA,
                        "PC3 -> diversity", "phi5", NA,
                        "logarea -> nmds1", "gamma1", NA,
                        "logarea -> nmds4", "gamma2",NA,
                        "logarea -> nmds5", "gamma4", NA,
                        "logarea -> diversity", "phi7", NA,
                        "logarea <-> logarea", NA, 1,
                        "diversity <-> diversity", "errordiv", NA,
                        "diversity -> S.obs", NA, 1, #This gives a scale to the latent variable by fixing the contribution of one of the indicators
                        "diversity -> shannon", "zeta2", NA,
                        "S.obs <-> S.obs", "error5", NA,
                        "shannon <-> shannon", "error7", NA),
                      ncol=3, byrow=TRUE)
correlations <- correlations[-8,-8]# This eliminates the column and row associated to S.chao1 in the correlation matrix. Do not repeat this command twice, as it will erase another column and row the second time.
sem.initial <- sem(mod.initial, correlations, 147)
summary(sem.initial)# Notice that model fit is still poor.

# We can try looking at the modification indices for this model:
mod.indices(sem.initial)
# The largest modification index involves a relationship from shannon to nmds1. However, we do not have any a priori reason to think that shannon diversity is causing a change in nmds1. So we incorporate this relationship as a simple covariance term that is not explained within our model by adding to the previous path one that has the reciprocal relationship nmds1 <-> shannon:

mod.initial <- matrix(c(
                        "PC1 -> nmds1", "alpha1", NA,
                        "PC1 -> nmds4", "alpha2", NA,
                        "PC1 -> nmds5", "alpha3", NA,
                        "PC1 <-> PC1", NA, 1,# the variance of exogenous variables was fixed to 1
                        "PC3 -> nmds1", "beta1", NA,
                        "PC3 -> nmds4", "beta2", NA,
                        "PC3 -> nmds5", "beta4", NA,
                        "PC3 <-> PC3", NA, 1, 
                        "nmds1 <-> nmds1", "err1", NA,
                        "nmds4 <-> nmds4", "err2", NA,
                        "nmds5 <-> nmds5", "err3", NA,
                        "nmds1 -> diversity", "phi1", NA, 
                        "nmds4 -> diversity", "phi2", NA,
                        "nmds5 -> diversity", "phi3", NA,
                        "PC1 -> diversity", "phi4", NA,
                        "PC3 -> diversity", "phi5", NA,
                        "logarea -> nmds1", "gamma1", NA,
                        "logarea -> nmds4", "gamma2",NA,
                        "logarea -> nmds5", "gamma4", NA,
                        "logarea -> diversity", "phi7", NA,
                        "logarea <-> logarea", NA, 1,
                        "diversity <-> diversity", "errordiv", NA,
                        "diversity -> S.obs", NA, 1, 
                        "diversity -> shannon", "zeta2", NA,
                        "S.obs <-> S.obs", "error5", NA,
                        "shannon <-> shannon", "error7", NA,
                        "nmds1 <-> shannon","modindex1",NA),
                      ncol=3, byrow=TRUE)
sem.initial <- sem(mod.initial, correlations, 147)
summary(sem.initial)# Now model fit is just good enough. By following the modification indices a couple of steps forward, the model fit still improves significantly. 
mod.initial <- matrix(c(
                        "PC1 -> nmds1", "alpha1", NA,
                        "PC1 -> nmds4", "alpha2", NA,
                        "PC1 -> nmds5", "alpha3", NA,
                        "PC1 <-> PC1", NA, 1,# the variance of exogenous variables was fixed to 1
                        "PC3 -> nmds1", "beta1", NA,
                        "PC3 -> nmds4", "beta2", NA,
                        "PC3 -> nmds5", "beta4", NA,
                        "PC3 <-> PC3", NA, 1, 
                        "nmds1 <-> nmds1", "err1", NA,
                        "nmds4 <-> nmds4", "err2", NA,
                        "nmds5 <-> nmds5", "err3", NA,
                        "nmds1 -> diversity", "phi1", NA, 
                        "nmds4 -> diversity", "phi2", NA,
                        "nmds5 -> diversity", "phi3", NA,
                        "PC1 -> diversity", "phi4", NA,
                        "PC3 -> diversity", "phi5", NA,
                        "logarea -> nmds1", "gamma1", NA,
                        "logarea -> nmds4", "gamma2",NA,
                        "logarea -> nmds5", "gamma4", NA,
                        "logarea -> diversity", "phi7", NA,
                        "logarea <-> logarea", NA, 1,
                        "diversity <-> diversity", "errordiv", NA,
                        "diversity -> S.obs", NA, 1, 
                        "diversity -> shannon", "zeta2", NA,
                        "S.obs <-> S.obs", "error5", NA,
                        "shannon <-> shannon", "error7", NA,
                        "nmds1 <-> shannon","modindex1",NA,
                        "logarea <-> shannon","modindex2", NA,
                        "logarea <-> PC1", "modindex3", NA),
                      ncol=3, byrow=TRUE)
sem.initial <- sem(mod.initial, correlations, 147)
summary(sem.initial)
# Now let's see which is the less significant path in the model:
summary(sem.initial)$coeff[summary(sem.initial)$coeff[,4]==max(summary(sem.initial)$coeff[,4]),]
#You can start reducing the previous model just by adding a # sign at the beginning of the line that you want to program to ignore

# This is the model specification for the final model:
mod.final <- matrix(c(
                      "PC1 -> nmds4", "alpha2", NA,
                      "PC1 -> nmds5", "alpha3", NA,
                      "PC1 <-> PC1", NA, 1,# the variance of exogenous variables was fixed to 1
                      "PC3 -> nmds1", "beta1", NA,
                      "PC3 -> nmds4", "beta2", NA,
                      "PC3 <-> PC3", NA, 1, 
                      "nmds1 <-> nmds1", "err1", NA,
                      "nmds4 <-> nmds4", "err2", NA,
                      "nmds5 <-> nmds5", "err3", NA,
                      "nmds1 -> diversity", "phi1", NA, 
                      "nmds4 -> diversity", "phi2", NA,
                      "nmds5 -> diversity", "phi3", NA,
                      "logarea -> nmds4", "gamma2",NA,
                      "logarea -> nmds5", "gamma4", NA,
                      "logarea -> diversity", "phi7", NA,
                      "logarea <-> logarea", NA, 1,
                      "diversity <-> diversity", "errordiv", NA,
                      "diversity -> S.obs", NA, 1, 
                      "diversity -> shannon", "zeta1", NA,
                      "S.obs <-> S.obs", "error5", NA,
                      "nmds1 <-> shannon","modindex1", NA,#this term was added after cheking the modification indices
                      "logarea <-> shannon","modindex2", NA,#this term was added after cheking the modification indices
                      "logarea <-> PC1", "modindex3", NA,#this term was added after cheking the modification indices
                      "shannon <-> shannon", "error7", NA),
                    ncol=3, byrow=TRUE)
sem.final <- sem(mod.final, correlations, 147)
summary(sem.final)
summary(sem.final)$coeff[summary(sem.final)$coeff[,4]>=0.05,]# There are still some terms that are not significant. We do not want to delete the term S.obs <-> S.obs since this has to do with the reliability of S.obs as indicator for diversity (although we could fix it to 0), and eliminating other non-significant paths actually decreases model fit, so this was the last simplification we wanted to make. 
